#!/usr/bin/python3
from qtt import QTT
import itertools
import sys


ftzversion = False
if len(sys.argv) >= 2 and sys.argv[1] == 'ftzdaz':
    ftzversion = True


# Create a new QTT instance
timer = QTT(iterations=40000000, use_rdtscp=False)


# Add the headers and librarys we need for math operations
timer.add_include("sse_ops.h")

if ftzversion:
    timer.add_include("mmintrin.h")
    timer.add_include("emmintrin.h")
    timer.add_include("xmmintrin.h")
    timer.add_include("immintrin.h")

timer.add_library("sse_ops.so")
timer.add_library("-lm")

args_s = [0.0,
          1.0,
          255,
          256,
          257,
          1.0e-42,
          1.0e-41,
          1.0e-30,
          1.0e30,
          1.0e10]

args_d = [1.0e-320,
          1.0e-300,
          1.0e200]

ss_args = args_s
sd_args = args_s + args_d

ts_args = list(itertools.product(ss_args, ss_args))
td_args = list(itertools.product(sd_args, sd_args))


singles_cfunc = ["sse_ops_sqrt_sp"]
singled_cfunc = ["sse_ops_sqrt_dp"]

twins_cfunc = ["sse_ops_add_sp", "sse_ops_sub_sp",
               "sse_ops_mul_sp", "sse_ops_div_sp"]
twind_cfunc = ["sse_ops_add_dp", "sse_ops_sub_dp",
               "sse_ops_mul_dp", "sse_ops_div_dp"]


timer.add_c_test(cfunc=singles_cfunc,
                 typestring="float (float)", args=ss_args)
timer.add_c_test(cfunc=singled_cfunc,
                 typestring="double (double)", args=sd_args)

timer.add_c_test(cfunc=twins_cfunc,
                 typestring="float (float, float)", args=ts_args)
timer.add_c_test(cfunc=twind_cfunc,
                 typestring="double (double, double)", args=td_args)


if ftzversion:
    timer.add_setup("""
unsigned int fpreg = _mm_getcsr();
fpreg = fpreg | 0x0040 | _MM_FLUSH_ZERO_ON;
_mm_setcsr(fpreg);
    """)


# Build the test
timer.build()

# Now go run the locally created a.qtt!
